# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    apps,
    plans,
    users,
    forums,
    entries,
    members,
    invoices,
    messages,
    payments,
    products,
    webhooks,
    companies,
    reactions,
    shipments,
    transfers,
    app_builds,
    experiences,
    forum_posts,
    memberships,
    chat_channels,
    ledger_accounts,
    authorized_users,
    support_channels,
    checkout_configurations,
    course_lesson_interactions,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import WhopError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Whop", "AsyncWhop", "Client", "AsyncClient"]


class Whop(SyncAPIClient):
    apps: apps.AppsResource
    invoices: invoices.InvoicesResource
    course_lesson_interactions: course_lesson_interactions.CourseLessonInteractionsResource
    products: products.ProductsResource
    companies: companies.CompaniesResource
    webhooks: webhooks.WebhooksResource
    plans: plans.PlansResource
    entries: entries.EntriesResource
    forum_posts: forum_posts.ForumPostsResource
    transfers: transfers.TransfersResource
    ledger_accounts: ledger_accounts.LedgerAccountsResource
    memberships: memberships.MembershipsResource
    authorized_users: authorized_users.AuthorizedUsersResource
    app_builds: app_builds.AppBuildsResource
    shipments: shipments.ShipmentsResource
    checkout_configurations: checkout_configurations.CheckoutConfigurationsResource
    messages: messages.MessagesResource
    chat_channels: chat_channels.ChatChannelsResource
    users: users.UsersResource
    payments: payments.PaymentsResource
    support_channels: support_channels.SupportChannelsResource
    experiences: experiences.ExperiencesResource
    reactions: reactions.ReactionsResource
    members: members.MembersResource
    forums: forums.ForumsResource
    with_raw_response: WhopWithRawResponse
    with_streaming_response: WhopWithStreamedResponse

    # client options
    api_key: str
    webhook_key: str | None
    app_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        webhook_key: str | None = None,
        app_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Whop client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `WHOP_API_KEY`
        - `webhook_key` from `WHOP_WEBHOOK_SECRET`
        - `app_id` from `WHOP_APP_ID`
        """
        if api_key is None:
            api_key = os.environ.get("WHOP_API_KEY")
        if api_key is None:
            raise WhopError(
                "The api_key client option must be set either by passing api_key to the client or by setting the WHOP_API_KEY environment variable"
            )
        self.api_key = api_key

        if webhook_key is None:
            webhook_key = os.environ.get("WHOP_WEBHOOK_SECRET")
        self.webhook_key = webhook_key

        if app_id is None:
            app_id = os.environ.get("WHOP_APP_ID")
        if app_id is None:
            raise WhopError(
                "The app_id client option must be set either by passing app_id to the client or by setting the WHOP_APP_ID environment variable"
            )
        self.app_id = app_id

        if base_url is None:
            base_url = os.environ.get("WHOP_BASE_URL")
        if base_url is None:
            base_url = f"https://api.whop.com/api/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.apps = apps.AppsResource(self)
        self.invoices = invoices.InvoicesResource(self)
        self.course_lesson_interactions = course_lesson_interactions.CourseLessonInteractionsResource(self)
        self.products = products.ProductsResource(self)
        self.companies = companies.CompaniesResource(self)
        self.webhooks = webhooks.WebhooksResource(self)
        self.plans = plans.PlansResource(self)
        self.entries = entries.EntriesResource(self)
        self.forum_posts = forum_posts.ForumPostsResource(self)
        self.transfers = transfers.TransfersResource(self)
        self.ledger_accounts = ledger_accounts.LedgerAccountsResource(self)
        self.memberships = memberships.MembershipsResource(self)
        self.authorized_users = authorized_users.AuthorizedUsersResource(self)
        self.app_builds = app_builds.AppBuildsResource(self)
        self.shipments = shipments.ShipmentsResource(self)
        self.checkout_configurations = checkout_configurations.CheckoutConfigurationsResource(self)
        self.messages = messages.MessagesResource(self)
        self.chat_channels = chat_channels.ChatChannelsResource(self)
        self.users = users.UsersResource(self)
        self.payments = payments.PaymentsResource(self)
        self.support_channels = support_channels.SupportChannelsResource(self)
        self.experiences = experiences.ExperiencesResource(self)
        self.reactions = reactions.ReactionsResource(self)
        self.members = members.MembersResource(self)
        self.forums = forums.ForumsResource(self)
        self.with_raw_response = WhopWithRawResponse(self)
        self.with_streaming_response = WhopWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "X-Whop-App-Id": self.app_id,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        webhook_key: str | None = None,
        app_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            webhook_key=webhook_key or self.webhook_key,
            app_id=app_id or self.app_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncWhop(AsyncAPIClient):
    apps: apps.AsyncAppsResource
    invoices: invoices.AsyncInvoicesResource
    course_lesson_interactions: course_lesson_interactions.AsyncCourseLessonInteractionsResource
    products: products.AsyncProductsResource
    companies: companies.AsyncCompaniesResource
    webhooks: webhooks.AsyncWebhooksResource
    plans: plans.AsyncPlansResource
    entries: entries.AsyncEntriesResource
    forum_posts: forum_posts.AsyncForumPostsResource
    transfers: transfers.AsyncTransfersResource
    ledger_accounts: ledger_accounts.AsyncLedgerAccountsResource
    memberships: memberships.AsyncMembershipsResource
    authorized_users: authorized_users.AsyncAuthorizedUsersResource
    app_builds: app_builds.AsyncAppBuildsResource
    shipments: shipments.AsyncShipmentsResource
    checkout_configurations: checkout_configurations.AsyncCheckoutConfigurationsResource
    messages: messages.AsyncMessagesResource
    chat_channels: chat_channels.AsyncChatChannelsResource
    users: users.AsyncUsersResource
    payments: payments.AsyncPaymentsResource
    support_channels: support_channels.AsyncSupportChannelsResource
    experiences: experiences.AsyncExperiencesResource
    reactions: reactions.AsyncReactionsResource
    members: members.AsyncMembersResource
    forums: forums.AsyncForumsResource
    with_raw_response: AsyncWhopWithRawResponse
    with_streaming_response: AsyncWhopWithStreamedResponse

    # client options
    api_key: str
    webhook_key: str | None
    app_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        webhook_key: str | None = None,
        app_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncWhop client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `WHOP_API_KEY`
        - `webhook_key` from `WHOP_WEBHOOK_SECRET`
        - `app_id` from `WHOP_APP_ID`
        """
        if api_key is None:
            api_key = os.environ.get("WHOP_API_KEY")
        if api_key is None:
            raise WhopError(
                "The api_key client option must be set either by passing api_key to the client or by setting the WHOP_API_KEY environment variable"
            )
        self.api_key = api_key

        if webhook_key is None:
            webhook_key = os.environ.get("WHOP_WEBHOOK_SECRET")
        self.webhook_key = webhook_key

        if app_id is None:
            app_id = os.environ.get("WHOP_APP_ID")
        if app_id is None:
            raise WhopError(
                "The app_id client option must be set either by passing app_id to the client or by setting the WHOP_APP_ID environment variable"
            )
        self.app_id = app_id

        if base_url is None:
            base_url = os.environ.get("WHOP_BASE_URL")
        if base_url is None:
            base_url = f"https://api.whop.com/api/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.apps = apps.AsyncAppsResource(self)
        self.invoices = invoices.AsyncInvoicesResource(self)
        self.course_lesson_interactions = course_lesson_interactions.AsyncCourseLessonInteractionsResource(self)
        self.products = products.AsyncProductsResource(self)
        self.companies = companies.AsyncCompaniesResource(self)
        self.webhooks = webhooks.AsyncWebhooksResource(self)
        self.plans = plans.AsyncPlansResource(self)
        self.entries = entries.AsyncEntriesResource(self)
        self.forum_posts = forum_posts.AsyncForumPostsResource(self)
        self.transfers = transfers.AsyncTransfersResource(self)
        self.ledger_accounts = ledger_accounts.AsyncLedgerAccountsResource(self)
        self.memberships = memberships.AsyncMembershipsResource(self)
        self.authorized_users = authorized_users.AsyncAuthorizedUsersResource(self)
        self.app_builds = app_builds.AsyncAppBuildsResource(self)
        self.shipments = shipments.AsyncShipmentsResource(self)
        self.checkout_configurations = checkout_configurations.AsyncCheckoutConfigurationsResource(self)
        self.messages = messages.AsyncMessagesResource(self)
        self.chat_channels = chat_channels.AsyncChatChannelsResource(self)
        self.users = users.AsyncUsersResource(self)
        self.payments = payments.AsyncPaymentsResource(self)
        self.support_channels = support_channels.AsyncSupportChannelsResource(self)
        self.experiences = experiences.AsyncExperiencesResource(self)
        self.reactions = reactions.AsyncReactionsResource(self)
        self.members = members.AsyncMembersResource(self)
        self.forums = forums.AsyncForumsResource(self)
        self.with_raw_response = AsyncWhopWithRawResponse(self)
        self.with_streaming_response = AsyncWhopWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "X-Whop-App-Id": self.app_id,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        webhook_key: str | None = None,
        app_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            webhook_key=webhook_key or self.webhook_key,
            app_id=app_id or self.app_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class WhopWithRawResponse:
    def __init__(self, client: Whop) -> None:
        self.apps = apps.AppsResourceWithRawResponse(client.apps)
        self.invoices = invoices.InvoicesResourceWithRawResponse(client.invoices)
        self.course_lesson_interactions = course_lesson_interactions.CourseLessonInteractionsResourceWithRawResponse(
            client.course_lesson_interactions
        )
        self.products = products.ProductsResourceWithRawResponse(client.products)
        self.companies = companies.CompaniesResourceWithRawResponse(client.companies)
        self.plans = plans.PlansResourceWithRawResponse(client.plans)
        self.entries = entries.EntriesResourceWithRawResponse(client.entries)
        self.forum_posts = forum_posts.ForumPostsResourceWithRawResponse(client.forum_posts)
        self.transfers = transfers.TransfersResourceWithRawResponse(client.transfers)
        self.ledger_accounts = ledger_accounts.LedgerAccountsResourceWithRawResponse(client.ledger_accounts)
        self.memberships = memberships.MembershipsResourceWithRawResponse(client.memberships)
        self.authorized_users = authorized_users.AuthorizedUsersResourceWithRawResponse(client.authorized_users)
        self.app_builds = app_builds.AppBuildsResourceWithRawResponse(client.app_builds)
        self.shipments = shipments.ShipmentsResourceWithRawResponse(client.shipments)
        self.checkout_configurations = checkout_configurations.CheckoutConfigurationsResourceWithRawResponse(
            client.checkout_configurations
        )
        self.messages = messages.MessagesResourceWithRawResponse(client.messages)
        self.chat_channels = chat_channels.ChatChannelsResourceWithRawResponse(client.chat_channels)
        self.users = users.UsersResourceWithRawResponse(client.users)
        self.payments = payments.PaymentsResourceWithRawResponse(client.payments)
        self.support_channels = support_channels.SupportChannelsResourceWithRawResponse(client.support_channels)
        self.experiences = experiences.ExperiencesResourceWithRawResponse(client.experiences)
        self.reactions = reactions.ReactionsResourceWithRawResponse(client.reactions)
        self.members = members.MembersResourceWithRawResponse(client.members)
        self.forums = forums.ForumsResourceWithRawResponse(client.forums)


class AsyncWhopWithRawResponse:
    def __init__(self, client: AsyncWhop) -> None:
        self.apps = apps.AsyncAppsResourceWithRawResponse(client.apps)
        self.invoices = invoices.AsyncInvoicesResourceWithRawResponse(client.invoices)
        self.course_lesson_interactions = (
            course_lesson_interactions.AsyncCourseLessonInteractionsResourceWithRawResponse(
                client.course_lesson_interactions
            )
        )
        self.products = products.AsyncProductsResourceWithRawResponse(client.products)
        self.companies = companies.AsyncCompaniesResourceWithRawResponse(client.companies)
        self.plans = plans.AsyncPlansResourceWithRawResponse(client.plans)
        self.entries = entries.AsyncEntriesResourceWithRawResponse(client.entries)
        self.forum_posts = forum_posts.AsyncForumPostsResourceWithRawResponse(client.forum_posts)
        self.transfers = transfers.AsyncTransfersResourceWithRawResponse(client.transfers)
        self.ledger_accounts = ledger_accounts.AsyncLedgerAccountsResourceWithRawResponse(client.ledger_accounts)
        self.memberships = memberships.AsyncMembershipsResourceWithRawResponse(client.memberships)
        self.authorized_users = authorized_users.AsyncAuthorizedUsersResourceWithRawResponse(client.authorized_users)
        self.app_builds = app_builds.AsyncAppBuildsResourceWithRawResponse(client.app_builds)
        self.shipments = shipments.AsyncShipmentsResourceWithRawResponse(client.shipments)
        self.checkout_configurations = checkout_configurations.AsyncCheckoutConfigurationsResourceWithRawResponse(
            client.checkout_configurations
        )
        self.messages = messages.AsyncMessagesResourceWithRawResponse(client.messages)
        self.chat_channels = chat_channels.AsyncChatChannelsResourceWithRawResponse(client.chat_channels)
        self.users = users.AsyncUsersResourceWithRawResponse(client.users)
        self.payments = payments.AsyncPaymentsResourceWithRawResponse(client.payments)
        self.support_channels = support_channels.AsyncSupportChannelsResourceWithRawResponse(client.support_channels)
        self.experiences = experiences.AsyncExperiencesResourceWithRawResponse(client.experiences)
        self.reactions = reactions.AsyncReactionsResourceWithRawResponse(client.reactions)
        self.members = members.AsyncMembersResourceWithRawResponse(client.members)
        self.forums = forums.AsyncForumsResourceWithRawResponse(client.forums)


class WhopWithStreamedResponse:
    def __init__(self, client: Whop) -> None:
        self.apps = apps.AppsResourceWithStreamingResponse(client.apps)
        self.invoices = invoices.InvoicesResourceWithStreamingResponse(client.invoices)
        self.course_lesson_interactions = (
            course_lesson_interactions.CourseLessonInteractionsResourceWithStreamingResponse(
                client.course_lesson_interactions
            )
        )
        self.products = products.ProductsResourceWithStreamingResponse(client.products)
        self.companies = companies.CompaniesResourceWithStreamingResponse(client.companies)
        self.plans = plans.PlansResourceWithStreamingResponse(client.plans)
        self.entries = entries.EntriesResourceWithStreamingResponse(client.entries)
        self.forum_posts = forum_posts.ForumPostsResourceWithStreamingResponse(client.forum_posts)
        self.transfers = transfers.TransfersResourceWithStreamingResponse(client.transfers)
        self.ledger_accounts = ledger_accounts.LedgerAccountsResourceWithStreamingResponse(client.ledger_accounts)
        self.memberships = memberships.MembershipsResourceWithStreamingResponse(client.memberships)
        self.authorized_users = authorized_users.AuthorizedUsersResourceWithStreamingResponse(client.authorized_users)
        self.app_builds = app_builds.AppBuildsResourceWithStreamingResponse(client.app_builds)
        self.shipments = shipments.ShipmentsResourceWithStreamingResponse(client.shipments)
        self.checkout_configurations = checkout_configurations.CheckoutConfigurationsResourceWithStreamingResponse(
            client.checkout_configurations
        )
        self.messages = messages.MessagesResourceWithStreamingResponse(client.messages)
        self.chat_channels = chat_channels.ChatChannelsResourceWithStreamingResponse(client.chat_channels)
        self.users = users.UsersResourceWithStreamingResponse(client.users)
        self.payments = payments.PaymentsResourceWithStreamingResponse(client.payments)
        self.support_channels = support_channels.SupportChannelsResourceWithStreamingResponse(client.support_channels)
        self.experiences = experiences.ExperiencesResourceWithStreamingResponse(client.experiences)
        self.reactions = reactions.ReactionsResourceWithStreamingResponse(client.reactions)
        self.members = members.MembersResourceWithStreamingResponse(client.members)
        self.forums = forums.ForumsResourceWithStreamingResponse(client.forums)


class AsyncWhopWithStreamedResponse:
    def __init__(self, client: AsyncWhop) -> None:
        self.apps = apps.AsyncAppsResourceWithStreamingResponse(client.apps)
        self.invoices = invoices.AsyncInvoicesResourceWithStreamingResponse(client.invoices)
        self.course_lesson_interactions = (
            course_lesson_interactions.AsyncCourseLessonInteractionsResourceWithStreamingResponse(
                client.course_lesson_interactions
            )
        )
        self.products = products.AsyncProductsResourceWithStreamingResponse(client.products)
        self.companies = companies.AsyncCompaniesResourceWithStreamingResponse(client.companies)
        self.plans = plans.AsyncPlansResourceWithStreamingResponse(client.plans)
        self.entries = entries.AsyncEntriesResourceWithStreamingResponse(client.entries)
        self.forum_posts = forum_posts.AsyncForumPostsResourceWithStreamingResponse(client.forum_posts)
        self.transfers = transfers.AsyncTransfersResourceWithStreamingResponse(client.transfers)
        self.ledger_accounts = ledger_accounts.AsyncLedgerAccountsResourceWithStreamingResponse(client.ledger_accounts)
        self.memberships = memberships.AsyncMembershipsResourceWithStreamingResponse(client.memberships)
        self.authorized_users = authorized_users.AsyncAuthorizedUsersResourceWithStreamingResponse(
            client.authorized_users
        )
        self.app_builds = app_builds.AsyncAppBuildsResourceWithStreamingResponse(client.app_builds)
        self.shipments = shipments.AsyncShipmentsResourceWithStreamingResponse(client.shipments)
        self.checkout_configurations = checkout_configurations.AsyncCheckoutConfigurationsResourceWithStreamingResponse(
            client.checkout_configurations
        )
        self.messages = messages.AsyncMessagesResourceWithStreamingResponse(client.messages)
        self.chat_channels = chat_channels.AsyncChatChannelsResourceWithStreamingResponse(client.chat_channels)
        self.users = users.AsyncUsersResourceWithStreamingResponse(client.users)
        self.payments = payments.AsyncPaymentsResourceWithStreamingResponse(client.payments)
        self.support_channels = support_channels.AsyncSupportChannelsResourceWithStreamingResponse(
            client.support_channels
        )
        self.experiences = experiences.AsyncExperiencesResourceWithStreamingResponse(client.experiences)
        self.reactions = reactions.AsyncReactionsResourceWithStreamingResponse(client.reactions)
        self.members = members.AsyncMembersResourceWithStreamingResponse(client.members)
        self.forums = forums.AsyncForumsResourceWithStreamingResponse(client.forums)


Client = Whop

AsyncClient = AsyncWhop



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import authorized_user_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.authorized_user_roles import AuthorizedUserRoles
from ..types.authorized_user_list_response import AuthorizedUserListResponse
from ..types.authorized_user_retrieve_response import AuthorizedUserRetrieveResponse

__all__ = ["AuthorizedUsersResource", "AsyncAuthorizedUsersResource"]


class AuthorizedUsersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthorizedUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AuthorizedUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthorizedUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AuthorizedUsersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthorizedUserRetrieveResponse:
        """
        Retrieves a authorized user by ID

        Required permissions:

        - `company:authorized_user:read`
        - `member:email:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/authorized_users/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthorizedUserRetrieveResponse,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        role: Optional[AuthorizedUserRoles] | Omit = omit,
        user_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[AuthorizedUserListResponse]:
        """
        Lists authorized users

        Required permissions:

        - `company:authorized_user:read`
        - `member:email:read`

        Args:
          company_id: The ID of the company to list authorized users for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          role: Possible roles an authorized user can have

          user_id: Filter by the user ID to check if the user is an authorized user

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/authorized_users",
            page=SyncCursorPage[AuthorizedUserListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "role": role,
                        "user_id": user_id,
                    },
                    authorized_user_list_params.AuthorizedUserListParams,
                ),
            ),
            model=AuthorizedUserListResponse,
        )


class AsyncAuthorizedUsersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthorizedUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthorizedUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthorizedUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncAuthorizedUsersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthorizedUserRetrieveResponse:
        """
        Retrieves a authorized user by ID

        Required permissions:

        - `company:authorized_user:read`
        - `member:email:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/authorized_users/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthorizedUserRetrieveResponse,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        role: Optional[AuthorizedUserRoles] | Omit = omit,
        user_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AuthorizedUserListResponse, AsyncCursorPage[AuthorizedUserListResponse]]:
        """
        Lists authorized users

        Required permissions:

        - `company:authorized_user:read`
        - `member:email:read`

        Args:
          company_id: The ID of the company to list authorized users for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          role: Possible roles an authorized user can have

          user_id: Filter by the user ID to check if the user is an authorized user

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/authorized_users",
            page=AsyncCursorPage[AuthorizedUserListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "role": role,
                        "user_id": user_id,
                    },
                    authorized_user_list_params.AuthorizedUserListParams,
                ),
            ),
            model=AuthorizedUserListResponse,
        )


class AuthorizedUsersResourceWithRawResponse:
    def __init__(self, authorized_users: AuthorizedUsersResource) -> None:
        self._authorized_users = authorized_users

        self.retrieve = to_raw_response_wrapper(
            authorized_users.retrieve,
        )
        self.list = to_raw_response_wrapper(
            authorized_users.list,
        )


class AsyncAuthorizedUsersResourceWithRawResponse:
    def __init__(self, authorized_users: AsyncAuthorizedUsersResource) -> None:
        self._authorized_users = authorized_users

        self.retrieve = async_to_raw_response_wrapper(
            authorized_users.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            authorized_users.list,
        )


class AuthorizedUsersResourceWithStreamingResponse:
    def __init__(self, authorized_users: AuthorizedUsersResource) -> None:
        self._authorized_users = authorized_users

        self.retrieve = to_streamed_response_wrapper(
            authorized_users.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            authorized_users.list,
        )


class AsyncAuthorizedUsersResourceWithStreamingResponse:
    def __init__(self, authorized_users: AsyncAuthorizedUsersResource) -> None:
        self._authorized_users = authorized_users

        self.retrieve = async_to_streamed_response_wrapper(
            authorized_users.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            authorized_users.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .shared.currency import Currency
from .shared.plan_type import PlanType
from .shared.visibility import Visibility
from .shared.release_method import ReleaseMethod

__all__ = ["PlanListResponse", "Company", "Invoice", "Product"]


class Company(BaseModel):
    id: str
    """The ID (tag) of the company."""

    title: str
    """The title of the company."""


class Invoice(BaseModel):
    id: str
    """The ID of the invoice."""


class Product(BaseModel):
    id: str
    """The internal ID of the public product."""

    title: str
    """The title of the product. Use for Whop 4.0."""


class PlanListResponse(BaseModel):
    id: str
    """The internal ID of the plan."""

    billing_period: Optional[int] = None
    """The interval at which the plan charges (renewal plans)."""

    company: Optional[Company] = None
    """The company for the plan."""

    created_at: datetime
    """When the plan was created."""

    currency: Currency
    """The respective currency identifier for the plan."""

    description: Optional[str] = None
    """The description of the plan."""

    expiration_days: Optional[int] = None
    """The interval at which the plan charges (expiration plans)."""

    initial_price: float
    """The price a person has to pay for a plan on the initial purchase."""

    internal_notes: Optional[str] = None
    """A personal description or notes section for the business."""

    invoice: Optional[Invoice] = None
    """The invoice associated with this plan."""

    member_count: Optional[int] = None
    """The number of members for the plan."""

    plan_type: PlanType
    """Indicates if the plan is a one time payment or recurring."""

    product: Optional[Product] = None
    """The access pass for the plan."""

    purchase_url: str
    """The direct link to purchase the product."""

    release_method: ReleaseMethod
    """This is the release method the business uses to sell this plan."""

    renewal_price: float
    """The price a person has to pay for a plan on the renewal purchase."""

    trial_period_days: Optional[int] = None
    """The number of free trial days added before a renewal plan."""

    updated_at: datetime
    """When the plan was last updated."""

    visibility: Visibility
    """Shows or hides the plan from public/business view."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, TypedDict

from .shared.app_statuses import AppStatuses

__all__ = ["AppUpdateParams", "Icon"]


class AppUpdateParams(TypedDict, total=False):
    app_store_description: Optional[str]
    """The description of the app for the app store in-depth app view."""

    base_url: Optional[str]
    """The base production url of the app"""

    dashboard_path: Optional[str]
    """The path for the dashboard view of the app"""

    description: Optional[str]
    """The description of the app"""

    discover_path: Optional[str]
    """The path for the discover view of the app"""

    experience_path: Optional[str]
    """The path for the hub view of the app"""

    icon: Optional[Icon]
    """The icon for the app"""

    name: Optional[str]
    """The name of the app"""

    required_scopes: Optional[List[Literal["read_user"]]]
    """The scopes that the app will request off of users when a user installs the app."""

    status: Optional[AppStatuses]
    """The status of an experience interface"""


class Icon(TypedDict, total=False):
    id: Optional[str]
    """The ID of an existing attachment object.

    Use this when updating a resource and keeping a subset of the attachments. Don't
    use this unless you know what you're doing.
    """

    direct_upload_id: Optional[str]
    """This ID should be used the first time you upload an attachment.

    It is the ID of the direct upload that was created when uploading the file to S3
    via the mediaDirectUpload mutation.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["AppBuildStatuses"]

AppBuildStatuses: TypeAlias = Literal["draft", "pending", "approved", "rejected"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional

import httpx

from ..types import app_build_list_params, app_build_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.app_build import AppBuild
from ..types.shared.app_view_type import AppViewType
from ..types.app_build_list_response import AppBuildListResponse
from ..types.shared.app_build_statuses import AppBuildStatuses
from ..types.shared.app_build_platforms import AppBuildPlatforms

__all__ = ["AppBuildsResource", "AsyncAppBuildsResource"]


class AppBuildsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AppBuildsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AppBuildsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AppBuildsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AppBuildsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        attachment: app_build_create_params.Attachment,
        checksum: str,
        platform: AppBuildPlatforms,
        app_id: Optional[str] | Omit = omit,
        supported_app_view_types: Optional[List[AppViewType]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppBuild:
        """
        Creates a new app build

        Required permissions:

        - `developer:manage_builds`

        Args:
          attachment: Attachment input for the app build file. This should be an upload in .zip
              format. The zip should contain at least one main_js_bundle.hbc file and
              optionally an assets folder next to it.

          checksum: Checksum of the app build file. This is generated by the client and used to
              verify the integrity of the file that is submitted when un-packaged later on a
              device.

          platform: The platform of the app build (ios, android, web)

          app_id: The ID of the app to create a build for. By default the current app from the api
              key is used.

          supported_app_view_types: Supported app view types for the app build. A build can specify multiple view
              types, but should only specify ones that its code supports.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/app_builds",
            body=maybe_transform(
                {
                    "attachment": attachment,
                    "checksum": checksum,
                    "platform": platform,
                    "app_id": app_id,
                    "supported_app_view_types": supported_app_view_types,
                },
                app_build_create_params.AppBuildCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AppBuild,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppBuild:
        """
        Retrieves an app build by ID

        Required permissions:

        - `developer:manage_builds`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/app_builds/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AppBuild,
        )

    def list(
        self,
        *,
        app_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        platform: Optional[AppBuildPlatforms] | Omit = omit,
        status: Optional[AppBuildStatuses] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[AppBuildListResponse]:
        """
        Lists app builds for an app

        Required permissions:

        - `developer:manage_builds`

        Args:
          app_id: The ID of the app to filter app builds by

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          platform: The different platforms an app build can target.

          status: The different statuses an AppBuild can be in.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/app_builds",
            page=SyncCursorPage[AppBuildListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "app_id": app_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "platform": platform,
                        "status": status,
                    },
                    app_build_list_params.AppBuildListParams,
                ),
            ),
            model=AppBuildListResponse,
        )

    def promote(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppBuild:
        """
        Promotes an app build to production

        Required permissions:

        - `developer:manage_builds`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/app_builds/{id}/promote",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AppBuild,
        )


class AsyncAppBuildsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAppBuildsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAppBuildsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAppBuildsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncAppBuildsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        attachment: app_build_create_params.Attachment,
        checksum: str,
        platform: AppBuildPlatforms,
        app_id: Optional[str] | Omit = omit,
        supported_app_view_types: Optional[List[AppViewType]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppBuild:
        """
        Creates a new app build

        Required permissions:

        - `developer:manage_builds`

        Args:
          attachment: Attachment input for the app build file. This should be an upload in .zip
              format. The zip should contain at least one main_js_bundle.hbc file and
              optionally an assets folder next to it.

          checksum: Checksum of the app build file. This is generated by the client and used to
              verify the integrity of the file that is submitted when un-packaged later on a
              device.

          platform: The platform of the app build (ios, android, web)

          app_id: The ID of the app to create a build for. By default the current app from the api
              key is used.

          supported_app_view_types: Supported app view types for the app build. A build can specify multiple view
              types, but should only specify ones that its code supports.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/app_builds",
            body=await async_maybe_transform(
                {
                    "attachment": attachment,
                    "checksum": checksum,
                    "platform": platform,
                    "app_id": app_id,
                    "supported_app_view_types": supported_app_view_types,
                },
                app_build_create_params.AppBuildCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AppBuild,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppBuild:
        """
        Retrieves an app build by ID

        Required permissions:

        - `developer:manage_builds`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/app_builds/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AppBuild,
        )

    def list(
        self,
        *,
        app_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        platform: Optional[AppBuildPlatforms] | Omit = omit,
        status: Optional[AppBuildStatuses] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AppBuildListResponse, AsyncCursorPage[AppBuildListResponse]]:
        """
        Lists app builds for an app

        Required permissions:

        - `developer:manage_builds`

        Args:
          app_id: The ID of the app to filter app builds by

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          platform: The different platforms an app build can target.

          status: The different statuses an AppBuild can be in.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/app_builds",
            page=AsyncCursorPage[AppBuildListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "app_id": app_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "platform": platform,
                        "status": status,
                    },
                    app_build_list_params.AppBuildListParams,
                ),
            ),
            model=AppBuildListResponse,
        )

    async def promote(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AppBuild:
        """
        Promotes an app build to production

        Required permissions:

        - `developer:manage_builds`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/app_builds/{id}/promote",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AppBuild,
        )


class AppBuildsResourceWithRawResponse:
    def __init__(self, app_builds: AppBuildsResource) -> None:
        self._app_builds = app_builds

        self.create = to_raw_response_wrapper(
            app_builds.create,
        )
        self.retrieve = to_raw_response_wrapper(
            app_builds.retrieve,
        )
        self.list = to_raw_response_wrapper(
            app_builds.list,
        )
        self.promote = to_raw_response_wrapper(
            app_builds.promote,
        )


class AsyncAppBuildsResourceWithRawResponse:
    def __init__(self, app_builds: AsyncAppBuildsResource) -> None:
        self._app_builds = app_builds

        self.create = async_to_raw_response_wrapper(
            app_builds.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            app_builds.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            app_builds.list,
        )
        self.promote = async_to_raw_response_wrapper(
            app_builds.promote,
        )


class AppBuildsResourceWithStreamingResponse:
    def __init__(self, app_builds: AppBuildsResource) -> None:
        self._app_builds = app_builds

        self.create = to_streamed_response_wrapper(
            app_builds.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            app_builds.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            app_builds.list,
        )
        self.promote = to_streamed_response_wrapper(
            app_builds.promote,
        )


class AsyncAppBuildsResourceWithStreamingResponse:
    def __init__(self, app_builds: AsyncAppBuildsResource) -> None:
        self._app_builds = app_builds

        self.create = async_to_streamed_response_wrapper(
            app_builds.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            app_builds.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            app_builds.list,
        )
        self.promote = async_to_streamed_response_wrapper(
            app_builds.promote,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["ExperienceCreateParams"]


class ExperienceCreateParams(TypedDict, total=False):
    app_id: Required[str]
    """The ID of the app to create the experience for"""

    company_id: Required[str]
    """The ID of the company to create the experience for"""

    name: Optional[str]
    """The name of the experience"""

    section_id: Optional[str]
    """The ID of the section to create the experience in"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr
from .shared.direction import Direction
from .shared.entry_status import EntryStatus

__all__ = ["EntryListParams"]


class EntryListParams(TypedDict, total=False):
    company_id: Required[str]
    """The ID of the company"""

    after: Optional[str]
    """Returns the elements in the list that come after the specified cursor."""

    before: Optional[str]
    """Returns the elements in the list that come before the specified cursor."""

    direction: Optional[Direction]
    """The direction of the sort."""

    first: Optional[int]
    """Returns the first _n_ elements from the list."""

    last: Optional[int]
    """Returns the last _n_ elements from the list."""

    order: Optional[Literal["id", "created_at"]]
    """Which columns can be used to sort."""

    plan_ids: Optional[SequenceNotStr[str]]
    """The plan IDs to filter the entries by"""

    product_ids: Optional[SequenceNotStr[str]]
    """The product IDs to filter the entries by"""

    statuses: Optional[List[EntryStatus]]
    """The statuses to filter the entries by"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

from ._models import BaseModel
from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["CursorPagePageInfo", "SyncCursorPage", "AsyncCursorPage"]

_T = TypeVar("_T")


class CursorPagePageInfo(BaseModel):
    end_cursor: Optional[str] = None


class SyncCursorPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    page_info: Optional[CursorPagePageInfo] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        end_cursor = None
        if self.page_info is not None:
            if self.page_info.end_cursor is not None:
                end_cursor = self.page_info.end_cursor
        if not end_cursor:
            return None

        return PageInfo(params={"after": end_cursor})


class AsyncCursorPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    page_info: Optional[CursorPagePageInfo] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        end_cursor = None
        if self.page_info is not None:
            if self.page_info.end_cursor is not None:
                end_cursor = self.page_info.end_cursor
        if not end_cursor:
            return None

        return PageInfo(params={"after": end_cursor})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["CustomCta"]

CustomCta: TypeAlias = Literal[
    "get_access",
    "join",
    "order_now",
    "shop_now",
    "call_now",
    "donate_now",
    "contact_us",
    "sign_up",
    "subscribe",
    "purchase",
    "get_offer",
    "apply_now",
    "complete_order",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .shared.currency import Currency
from .shared.promo_type import PromoType
from .shared.receipt_status import ReceiptStatus
from .shared.membership_status import MembershipStatus
from .shared.friendly_receipt_status import FriendlyReceiptStatus

__all__ = [
    "PaymentListResponse",
    "BillingAddress",
    "Company",
    "Member",
    "Membership",
    "Plan",
    "Product",
    "PromoCode",
    "User",
]


class BillingAddress(BaseModel):
    city: Optional[str] = None
    """The city of the address."""

    country: Optional[str] = None
    """The country of the address."""

    line1: Optional[str] = None
    """The line 1 of the address."""

    line2: Optional[str] = None
    """The line 2 of the address."""

    name: Optional[str] = None
    """The name of the customer."""

    postal_code: Optional[str] = None
    """The postal code of the address."""

    state: Optional[str] = None
    """The state of the address."""


class Company(BaseModel):
    id: str
    """The ID of the company"""

    route: str
    """The slug/route of the company on the Whop site."""

    title: str
    """The written name of the company."""


class Member(BaseModel):
    id: str
    """The ID of the member"""

    phone: Optional[str] = None
    """The phone number for the member, if available."""


class Membership(BaseModel):
    id: str
    """The internal ID of the membership."""

    status: MembershipStatus
    """The state of the membership."""


class Plan(BaseModel):
    id: str
    """The internal ID of the plan."""


class Product(BaseModel):
    id: str
    """The internal ID of the public product."""

    route: str
    """The route of the product."""

    title: str
    """The title of the product. Use for Whop 4.0."""


class PromoCode(BaseModel):
    id: str
    """The ID of the promo."""

    amount_off: float
    """The amount off (% or flat amount) for the promo."""

    base_currency: Currency
    """The monetary currency of the promo code."""

    code: Optional[str] = None
    """The specific code used to apply the promo at checkout."""

    number_of_intervals: Optional[int] = None
    """The number of billing cycles the promo is applied for."""

    promo_type: PromoType
    """The type (% or flat amount) of the promo."""


class User(BaseModel):
    id: str
    """The internal ID of the user."""

    email: Optional[str] = None
    """The email of the user"""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class PaymentListResponse(BaseModel):
    id: str
    """The payment ID"""

    amount_after_fees: float
    """How much the payment is for after fees"""

    auto_refunded: bool
    """Whether this payment was auto refunded or not"""

    billing_address: Optional[BillingAddress] = None
    """The address of the user who made the payment."""

    billing_reason: Optional[str] = None
    """The billing reason"""

    card_brand: Optional[str] = None
    """The type of card used as the payment method."""

    card_last4: Optional[str] = None
    """The last 4 digits of the card used to make the payment."""

    company: Optional[Company] = None
    """The company for the payment."""

    created_at: datetime
    """The datetime the payment was created"""

    currency: Optional[Currency] = None
    """The available currencies on the platform"""

    dispute_alerted_at: Optional[datetime] = None
    """When an alert came in that this transaction will be disputed"""

    failure_message: Optional[str] = None
    """If the payment failed, the reason for the failure."""

    last_payment_attempt: Optional[datetime] = None
    """The time of the last payment attempt."""

    member: Optional[Member] = None
    """The member attached to this payment."""

    membership: Optional[Membership] = None
    """The membership attached to this payment."""

    paid_at: Optional[datetime] = None
    """The datetime the payment was paid"""

    payment_method_type: Optional[str] = None
    """Returns the type of payment method used for the payment, if available.

    Ex. klarna, affirm, card, cashapp
    """

    plan: Optional[Plan] = None
    """The plan attached to this payment."""

    product: Optional[Product] = None
    """The access pass attached to this payment."""

    promo_code: Optional[PromoCode] = None
    """The promo code used for this payment."""

    refundable: bool
    """Whether the payment can be refunded."""

    refunded_amount: Optional[float] = None
    """The payment refund amount(if applicable)."""

    refunded_at: Optional[datetime] = None
    """When the payment was refunded (if applicable)."""

    retryable: bool
    """Whether the payment can be retried."""

    status: Optional[ReceiptStatus] = None
    """The status of a receipt"""

    substatus: FriendlyReceiptStatus
    """The friendly status of the payment."""

    subtotal: Optional[float] = None
    """The subtotal to show to the creator (excluding buyer fees)."""

    total: Optional[float] = None
    """The total to show to the creator (excluding buyer fees)."""

    usd_total: Optional[float] = None
    """The total in USD to show to the creator (excluding buyer fees)."""

    user: Optional[User] = None
    """The user that made this payment."""

    voidable: bool
    """Whether the payment can be voided."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ExperienceListResponse", "App", "AppIcon", "Company", "Image"]


class AppIcon(BaseModel):
    url: Optional[str] = None
    """This is the URL you use to render optimized attachments on the client.

    This should be used for apps.
    """


class App(BaseModel):
    id: str
    """The ID of the app"""

    icon: Optional[AppIcon] = None
    """The icon for the app.

    This icon is shown on discovery, on the product page, on checkout, and as a
    default icon for the experiences.
    """

    name: str
    """The name of the app"""


class Company(BaseModel):
    id: str
    """The ID (tag) of the company."""

    route: str
    """The slug/route of the company on the Whop site."""

    title: str
    """The title of the company."""


class Image(BaseModel):
    url: Optional[str] = None
    """This is the URL you use to render optimized attachments on the client.

    This should be used for apps.
    """


class ExperienceListResponse(BaseModel):
    id: str
    """The unique ID representing this experience"""

    app: App
    """The experience interface for this experience."""

    company: Company
    """The company that owns this experience."""

    created_at: datetime
    """The timestamp of when this experience was created."""

    image: Optional[Image] = None
    """The logo for the experience."""

    name: str
    """The written name of the description."""

    order: Optional[str] = None
    """The order of the experience in the section"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .shared.shipment_status import ShipmentStatus
from .shared.shipment_carrier import ShipmentCarrier
from .shared.shipment_substatus import ShipmentSubstatus

__all__ = ["ShipmentListResponse", "Payment"]


class Payment(BaseModel):
    id: str
    """The payment ID"""


class ShipmentListResponse(BaseModel):
    id: str
    """The ID of the shipment"""

    carrier: ShipmentCarrier
    """The carrier of the shipment"""

    created_at: datetime
    """The date and time the shipment was created"""

    delivery_estimate: Optional[datetime] = None
    """The delivery estimate of the shipment"""

    payment: Payment
    """The payment of the shipment"""

    service: Optional[str] = None
    """The service of the shipment"""

    status: ShipmentStatus
    """The status of the shipment"""

    substatus: Optional[ShipmentSubstatus] = None
    """The substatus of a shipment"""

    tracking_code: str
    """The tracking code of the shipment"""

    updated_at: datetime
    """The date and time the shipment was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .currency import Currency
from ..._models import BaseModel
from .membership_status import MembershipStatus

__all__ = ["Membership", "Company", "Member", "Plan", "PromoCode", "User"]


class Company(BaseModel):
    id: str
    """The ID (tag) of the company."""

    title: str
    """The title of the company."""


class Member(BaseModel):
    id: str
    """The ID of the member"""


class Plan(BaseModel):
    id: str
    """The internal ID of the plan."""


class PromoCode(BaseModel):
    id: str
    """The ID of the promo."""


class User(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class Membership(BaseModel):
    id: str
    """The ID of the membership"""

    cancel_at_period_end: bool
    """Whether this Membership is set to cancel at the end of the current billing
    cycle.

    Only applies for memberships that have a renewal plan.
    """

    canceled_at: Optional[datetime] = None
    """The epoch timestamp of when the customer initiated a cancellation."""

    cancellation_reason: Optional[str] = None
    """The reason that the member canceled the membership (filled out by the member)."""

    company: Company
    """The Company this Membership belongs to."""

    created_at: datetime
    """The timestamp, in seconds, that this Membership was created at."""

    currency: Optional[Currency] = None
    """The available currencies on the platform"""

    license_key: Optional[str] = None
    """The license key for this Membership.

    This is only present if the membership grants access to an instance of the Whop
    Software app.
    """

    manage_url: Optional[str] = None
    """The URL for the customer to manage their membership."""

    member: Optional[Member] = None
    """The Member that this Membership belongs to."""

    metadata: Dict[str, object]
    """A JSON object used to store software licensing information. Ex. HWID"""

    payment_collection_paused: bool
    """Whether the membership's payments are currently paused."""

    plan: Plan
    """The Plan this Membership is for."""

    promo_code: Optional[PromoCode] = None
    """The Promo Code that is currently applied to this Membership."""

    renewal_period_end: Optional[datetime] = None
    """
    The timestamp in seconds at which the current billing cycle for this
    subscription ends. Only applies for memberships that have a renewal plan.
    """

    renewal_period_start: Optional[datetime] = None
    """
    The timestamp in seconds at which the current billing cycle for this
    subscription start. Only applies for memberships that have a renewal plan.
    """

    status: MembershipStatus
    """The status of the membership."""

    updated_at: datetime
    """A timestamp of when the membership was last updated"""

    user: Optional[User] = None
    """The user this membership belongs to"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .shared.currency import Currency

__all__ = ["TransferCreateParams"]


class TransferCreateParams(TypedDict, total=False):
    amount: Required[float]
    """The amount to withdraw"""

    currency: Required[Currency]
    """The currency that is being withdrawn."""

    destination_id: Required[str]
    """
    The ID of the destination account which will receive the funds (either a User
    ID, Company ID, or LedgerAccount ID)
    """

    origin_id: Required[str]
    """
    The ID of the origin account which will send the funds (either a User ID,
    Company ID, or LedgerAccount ID)
    """

    idempotence_key: Optional[str]
    """A unique key to ensure idempotence. Use a UUID or similar."""

    notes: Optional[str]
    """Notes for the transfer. Maximum of 50 characters."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional

import httpx

from ..types import forum_post_list_params, forum_post_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.currency import Currency
from ..types.shared.forum_post import ForumPost
from ..types.forum_post_list_response import ForumPostListResponse

__all__ = ["ForumPostsResource", "AsyncForumPostsResource"]


class ForumPostsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ForumPostsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return ForumPostsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ForumPostsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return ForumPostsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        experience_id: str,
        attachments: Optional[Iterable[forum_post_create_params.Attachment]] | Omit = omit,
        content: Optional[str] | Omit = omit,
        is_mention: Optional[bool] | Omit = omit,
        parent_id: Optional[str] | Omit = omit,
        paywall_amount: Optional[float] | Omit = omit,
        paywall_currency: Optional[Currency] | Omit = omit,
        pinned: Optional[bool] | Omit = omit,
        poll: Optional[forum_post_create_params.Poll] | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ForumPost:
        """
        Create a new forum post

        Required permissions:

        - `forum:post:create`

        Args:
          experience_id: The experience to create this post in

          attachments: The attachments for this post

          content: This is the main body of the post in Markdown format. Hidden if paywalled and
              user hasn't purchased access to it.

          is_mention: This is used to determine if the post should be sent as a 'mention' notification
              to all of the users who are in the experience. This means that anyone with
              'mentions' enabled will receive a notification about this post.

          parent_id: The ID of the parent post. Set it to the ID of the post you want to comment on
              or don't include it if its a top level post.

          paywall_amount: The amount to paywall this post by. A paywalled post requires the user to
              purchase it in order to view its content.

          paywall_currency: The available currencies on the platform

          pinned: Whether the post should be pinned

          poll: The poll for this post

          title: The title of the post. Only visible if paywalled.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/forum_posts",
            body=maybe_transform(
                {
                    "experience_id": experience_id,
                    "attachments": attachments,
                    "content": content,
                    "is_mention": is_mention,
                    "parent_id": parent_id,
                    "paywall_amount": paywall_amount,
                    "paywall_currency": paywall_currency,
                    "pinned": pinned,
                    "poll": poll,
                    "title": title,
                },
                forum_post_create_params.ForumPostCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForumPost,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ForumPost:
        """
        Retrieves a forum post by ID

        Required permissions:

        - `forum:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/forum_posts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForumPost,
        )

    def list(
        self,
        *,
        experience_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        parent_id: Optional[str] | Omit = omit,
        pinned: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[ForumPostListResponse]:
        """
        Lists forum posts

        Required permissions:

        - `forum:read`

        Args:
          experience_id: The ID of the experience to list forum posts for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          parent_id: The ID of the parent post to list forum post comments for

          pinned: Set to true to only return pinned posts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/forum_posts",
            page=SyncCursorPage[ForumPostListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "experience_id": experience_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "parent_id": parent_id,
                        "pinned": pinned,
                    },
                    forum_post_list_params.ForumPostListParams,
                ),
            ),
            model=ForumPostListResponse,
        )


class AsyncForumPostsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncForumPostsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncForumPostsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncForumPostsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncForumPostsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        experience_id: str,
        attachments: Optional[Iterable[forum_post_create_params.Attachment]] | Omit = omit,
        content: Optional[str] | Omit = omit,
        is_mention: Optional[bool] | Omit = omit,
        parent_id: Optional[str] | Omit = omit,
        paywall_amount: Optional[float] | Omit = omit,
        paywall_currency: Optional[Currency] | Omit = omit,
        pinned: Optional[bool] | Omit = omit,
        poll: Optional[forum_post_create_params.Poll] | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ForumPost:
        """
        Create a new forum post

        Required permissions:

        - `forum:post:create`

        Args:
          experience_id: The experience to create this post in

          attachments: The attachments for this post

          content: This is the main body of the post in Markdown format. Hidden if paywalled and
              user hasn't purchased access to it.

          is_mention: This is used to determine if the post should be sent as a 'mention' notification
              to all of the users who are in the experience. This means that anyone with
              'mentions' enabled will receive a notification about this post.

          parent_id: The ID of the parent post. Set it to the ID of the post you want to comment on
              or don't include it if its a top level post.

          paywall_amount: The amount to paywall this post by. A paywalled post requires the user to
              purchase it in order to view its content.

          paywall_currency: The available currencies on the platform

          pinned: Whether the post should be pinned

          poll: The poll for this post

          title: The title of the post. Only visible if paywalled.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/forum_posts",
            body=await async_maybe_transform(
                {
                    "experience_id": experience_id,
                    "attachments": attachments,
                    "content": content,
                    "is_mention": is_mention,
                    "parent_id": parent_id,
                    "paywall_amount": paywall_amount,
                    "paywall_currency": paywall_currency,
                    "pinned": pinned,
                    "poll": poll,
                    "title": title,
                },
                forum_post_create_params.ForumPostCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForumPost,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ForumPost:
        """
        Retrieves a forum post by ID

        Required permissions:

        - `forum:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/forum_posts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForumPost,
        )

    def list(
        self,
        *,
        experience_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        parent_id: Optional[str] | Omit = omit,
        pinned: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ForumPostListResponse, AsyncCursorPage[ForumPostListResponse]]:
        """
        Lists forum posts

        Required permissions:

        - `forum:read`

        Args:
          experience_id: The ID of the experience to list forum posts for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          parent_id: The ID of the parent post to list forum post comments for

          pinned: Set to true to only return pinned posts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/forum_posts",
            page=AsyncCursorPage[ForumPostListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "experience_id": experience_id,
                        "after": after,
                        "before": before,
                        "first": first,
                        "last": last,
                        "parent_id": parent_id,
                        "pinned": pinned,
                    },
                    forum_post_list_params.ForumPostListParams,
                ),
            ),
            model=ForumPostListResponse,
        )


class ForumPostsResourceWithRawResponse:
    def __init__(self, forum_posts: ForumPostsResource) -> None:
        self._forum_posts = forum_posts

        self.create = to_raw_response_wrapper(
            forum_posts.create,
        )
        self.retrieve = to_raw_response_wrapper(
            forum_posts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            forum_posts.list,
        )


class AsyncForumPostsResourceWithRawResponse:
    def __init__(self, forum_posts: AsyncForumPostsResource) -> None:
        self._forum_posts = forum_posts

        self.create = async_to_raw_response_wrapper(
            forum_posts.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            forum_posts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            forum_posts.list,
        )


class ForumPostsResourceWithStreamingResponse:
    def __init__(self, forum_posts: ForumPostsResource) -> None:
        self._forum_posts = forum_posts

        self.create = to_streamed_response_wrapper(
            forum_posts.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            forum_posts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            forum_posts.list,
        )


class AsyncForumPostsResourceWithStreamingResponse:
    def __init__(self, forum_posts: AsyncForumPostsResource) -> None:
        self._forum_posts = forum_posts

        self.create = async_to_streamed_response_wrapper(
            forum_posts.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            forum_posts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            forum_posts.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .entry_denied_webhook_event import EntryDeniedWebhookEvent
from .invoice_paid_webhook_event import InvoicePaidWebhookEvent
from .entry_created_webhook_event import EntryCreatedWebhookEvent
from .entry_deleted_webhook_event import EntryDeletedWebhookEvent
from .entry_approved_webhook_event import EntryApprovedWebhookEvent
from .invoice_voided_webhook_event import InvoiceVoidedWebhookEvent
from .payment_failed_webhook_event import PaymentFailedWebhookEvent
from .invoice_created_webhook_event import InvoiceCreatedWebhookEvent
from .payment_pending_webhook_event import PaymentPendingWebhookEvent
from .invoice_past_due_webhook_event import InvoicePastDueWebhookEvent
from .payment_succeeded_webhook_event import PaymentSucceededWebhookEvent
from .membership_activated_webhook_event import MembershipActivatedWebhookEvent
from .membership_deactivated_webhook_event import MembershipDeactivatedWebhookEvent
from .course_lesson_interaction_completed_webhook_event import CourseLessonInteractionCompletedWebhookEvent

__all__ = ["UnwrapWebhookEvent"]

UnwrapWebhookEvent: TypeAlias = Union[
    InvoiceCreatedWebhookEvent,
    InvoicePaidWebhookEvent,
    InvoicePastDueWebhookEvent,
    InvoiceVoidedWebhookEvent,
    MembershipActivatedWebhookEvent,
    MembershipDeactivatedWebhookEvent,
    EntryCreatedWebhookEvent,
    EntryApprovedWebhookEvent,
    EntryDeniedWebhookEvent,
    EntryDeletedWebhookEvent,
    CourseLessonInteractionCompletedWebhookEvent,
    PaymentSucceededWebhookEvent,
    PaymentFailedWebhookEvent,
    PaymentPendingWebhookEvent,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["MembershipPauseParams"]


class MembershipPauseParams(TypedDict, total=False):
    void_payments: Optional[bool]
    """
    Whether to void past_due payments associated with the membership to prevent
    future payment attempts.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["ReceiptStatus"]

ReceiptStatus: TypeAlias = Literal["draft", "open", "paid", "pending", "uncollectible", "unresolved", "void"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ShipmentStatus"]

ShipmentStatus: TypeAlias = Literal[
    "unknown",
    "pre_transit",
    "in_transit",
    "out_for_delivery",
    "delivered",
    "available_for_pickup",
    "return_to_sender",
    "failure",
    "cancelled",
    "error",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import shipment_list_params, shipment_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.shipment import Shipment
from ..types.shipment_list_response import ShipmentListResponse

__all__ = ["ShipmentsResource", "AsyncShipmentsResource"]


class ShipmentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ShipmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return ShipmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ShipmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return ShipmentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        company_id: str,
        payment_id: str,
        tracking_code: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Shipment:
        """
        Creates a new shipment

        Required permissions:

        - `shipment:create`
        - `payment:basic:read`

        Args:
          company_id: The ID of the company to create the shipment for

          payment_id: The ID of the payment to create the shipment for

          tracking_code: The tracking code for the shipment

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/shipments",
            body=maybe_transform(
                {
                    "company_id": company_id,
                    "payment_id": payment_id,
                    "tracking_code": tracking_code,
                },
                shipment_create_params.ShipmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Shipment,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Shipment:
        """
        Retrieves a shipment by ID

        Required permissions:

        - `shipment:basic:read`
        - `payment:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/shipments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Shipment,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        company_id: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        payment_id: Optional[str] | Omit = omit,
        user_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[ShipmentListResponse]:
        """
        Lists shipments for a payment

        Required permissions:

        - `shipment:basic:read`
        - `payment:basic:read`

        Args:
          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          company_id: The ID of the company

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          payment_id: The ID of the payment

          user_id: The ID of the user

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/shipments",
            page=SyncCursorPage[ShipmentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "company_id": company_id,
                        "first": first,
                        "last": last,
                        "payment_id": payment_id,
                        "user_id": user_id,
                    },
                    shipment_list_params.ShipmentListParams,
                ),
            ),
            model=ShipmentListResponse,
        )


class AsyncShipmentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncShipmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncShipmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncShipmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncShipmentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        company_id: str,
        payment_id: str,
        tracking_code: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Shipment:
        """
        Creates a new shipment

        Required permissions:

        - `shipment:create`
        - `payment:basic:read`

        Args:
          company_id: The ID of the company to create the shipment for

          payment_id: The ID of the payment to create the shipment for

          tracking_code: The tracking code for the shipment

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/shipments",
            body=await async_maybe_transform(
                {
                    "company_id": company_id,
                    "payment_id": payment_id,
                    "tracking_code": tracking_code,
                },
                shipment_create_params.ShipmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Shipment,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Shipment:
        """
        Retrieves a shipment by ID

        Required permissions:

        - `shipment:basic:read`
        - `payment:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/shipments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Shipment,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        company_id: Optional[str] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        payment_id: Optional[str] | Omit = omit,
        user_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ShipmentListResponse, AsyncCursorPage[ShipmentListResponse]]:
        """
        Lists shipments for a payment

        Required permissions:

        - `shipment:basic:read`
        - `payment:basic:read`

        Args:
          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          company_id: The ID of the company

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          payment_id: The ID of the payment

          user_id: The ID of the user

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/shipments",
            page=AsyncCursorPage[ShipmentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "company_id": company_id,
                        "first": first,
                        "last": last,
                        "payment_id": payment_id,
                        "user_id": user_id,
                    },
                    shipment_list_params.ShipmentListParams,
                ),
            ),
            model=ShipmentListResponse,
        )


class ShipmentsResourceWithRawResponse:
    def __init__(self, shipments: ShipmentsResource) -> None:
        self._shipments = shipments

        self.create = to_raw_response_wrapper(
            shipments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            shipments.retrieve,
        )
        self.list = to_raw_response_wrapper(
            shipments.list,
        )


class AsyncShipmentsResourceWithRawResponse:
    def __init__(self, shipments: AsyncShipmentsResource) -> None:
        self._shipments = shipments

        self.create = async_to_raw_response_wrapper(
            shipments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            shipments.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            shipments.list,
        )


class ShipmentsResourceWithStreamingResponse:
    def __init__(self, shipments: ShipmentsResource) -> None:
        self._shipments = shipments

        self.create = to_streamed_response_wrapper(
            shipments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            shipments.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            shipments.list,
        )


class AsyncShipmentsResourceWithStreamingResponse:
    def __init__(self, shipments: AsyncShipmentsResource) -> None:
        self._shipments = shipments

        self.create = async_to_streamed_response_wrapper(
            shipments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            shipments.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            shipments.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["MemberMostRecentActions"]

MemberMostRecentActions: TypeAlias = Literal[
    "canceling",
    "churned",
    "finished_split_pay",
    "paused",
    "paid_subscriber",
    "paid_once",
    "expiring",
    "joined",
    "drafted",
    "left",
    "trialing",
    "pending_entry",
    "renewing",
    "past_due",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr
from .shared.direction import Direction
from .shared.plan_type import PlanType
from .shared.release_method import ReleaseMethod
from .shared.visibility_filter import VisibilityFilter

__all__ = ["PlanListParams"]


class PlanListParams(TypedDict, total=False):
    company_id: Required[str]
    """The ID of the company"""

    after: Optional[str]
    """Returns the elements in the list that come after the specified cursor."""

    before: Optional[str]
    """Returns the elements in the list that come before the specified cursor."""

    direction: Optional[Direction]
    """The direction of the sort."""

    first: Optional[int]
    """Returns the first _n_ elements from the list."""

    last: Optional[int]
    """Returns the last _n_ elements from the list."""

    order: Optional[Literal["id", "active_members_count", "created_at", "internal_notes", "expires_at"]]
    """The ways a relation of Plans can be ordered"""

    plan_types: Optional[List[PlanType]]
    """The plan type to filter the plans by"""

    product_ids: Optional[SequenceNotStr[str]]
    """The product IDs to filter the plans by"""

    release_methods: Optional[List[ReleaseMethod]]
    """The release method to filter the plans by"""

    visibilities: Optional[List[VisibilityFilter]]
    """The visibility to filter the plans by"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

from .shared.direction import Direction
from .shared.access_pass_type import AccessPassType
from .shared.visibility_filter import VisibilityFilter

__all__ = ["ProductListParams"]


class ProductListParams(TypedDict, total=False):
    company_id: Required[str]
    """The ID of the company to filter products by"""

    after: Optional[str]
    """Returns the elements in the list that come after the specified cursor."""

    before: Optional[str]
    """Returns the elements in the list that come before the specified cursor."""

    direction: Optional[Direction]
    """The direction of the sort."""

    first: Optional[int]
    """Returns the first _n_ elements from the list."""

    last: Optional[int]
    """Returns the last _n_ elements from the list."""

    order: Optional[Literal["active_memberships_count", "created_at", "usd_gmv", "usd_gmv_30_days"]]
    """The ways a relation of AccessPasses can be ordered"""

    product_types: Optional[List[AccessPassType]]
    """The type of products to filter by"""

    visibilities: Optional[List[VisibilityFilter]]
    """The visibility of the products to filter by"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import PaymentListResponse
from whop_sdk._utils import parse_datetime
from whop_sdk.pagination import SyncCursorPage, AsyncCursorPage
from whop_sdk.types.shared import Payment

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPayments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        payment = client.payments.retrieve(
            "pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.payments.with_raw_response.retrieve(
            "pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.payments.with_streaming_response.retrieve(
            "pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.payments.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Whop) -> None:
        payment = client.payments.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(SyncCursorPage[PaymentListResponse], payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Whop) -> None:
        payment = client.payments.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            billing_reasons=["subscription_create"],
            created_after=parse_datetime("2023-12-01T05:00:00.401Z"),
            created_before=parse_datetime("2023-12-01T05:00:00.401Z"),
            currencies=["usd"],
            direction="asc",
            first=42,
            include_free=True,
            last=42,
            order="final_amount",
            plan_ids=["string"],
            product_ids=["string"],
            statuses=["draft"],
            substatuses=["auto_refunded"],
        )
        assert_matches_type(SyncCursorPage[PaymentListResponse], payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Whop) -> None:
        response = client.payments.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(SyncCursorPage[PaymentListResponse], payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Whop) -> None:
        with client.payments.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(SyncCursorPage[PaymentListResponse], payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_refund(self, client: Whop) -> None:
        payment = client.payments.refund(
            id="pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_refund_with_all_params(self, client: Whop) -> None:
        payment = client.payments.refund(
            id="pay_xxxxxxxxxxxxxx",
            partial_amount=6.9,
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_refund(self, client: Whop) -> None:
        response = client.payments.with_raw_response.refund(
            id="pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_refund(self, client: Whop) -> None:
        with client.payments.with_streaming_response.refund(
            id="pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_refund(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.payments.with_raw_response.refund(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retry(self, client: Whop) -> None:
        payment = client.payments.retry(
            "pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retry(self, client: Whop) -> None:
        response = client.payments.with_raw_response.retry(
            "pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retry(self, client: Whop) -> None:
        with client.payments.with_streaming_response.retry(
            "pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retry(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.payments.with_raw_response.retry(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_void(self, client: Whop) -> None:
        payment = client.payments.void(
            "pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_void(self, client: Whop) -> None:
        response = client.payments.with_raw_response.void(
            "pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_void(self, client: Whop) -> None:
        with client.payments.with_streaming_response.void(
            "pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_void(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.payments.with_raw_response.void(
                "",
            )


class TestAsyncPayments:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        payment = await async_client.payments.retrieve(
            "pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.payments.with_raw_response.retrieve(
            "pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.payments.with_streaming_response.retrieve(
            "pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.payments.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncWhop) -> None:
        payment = await async_client.payments.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(AsyncCursorPage[PaymentListResponse], payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncWhop) -> None:
        payment = await async_client.payments.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            billing_reasons=["subscription_create"],
            created_after=parse_datetime("2023-12-01T05:00:00.401Z"),
            created_before=parse_datetime("2023-12-01T05:00:00.401Z"),
            currencies=["usd"],
            direction="asc",
            first=42,
            include_free=True,
            last=42,
            order="final_amount",
            plan_ids=["string"],
            product_ids=["string"],
            statuses=["draft"],
            substatuses=["auto_refunded"],
        )
        assert_matches_type(AsyncCursorPage[PaymentListResponse], payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWhop) -> None:
        response = await async_client.payments.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(AsyncCursorPage[PaymentListResponse], payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWhop) -> None:
        async with async_client.payments.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(AsyncCursorPage[PaymentListResponse], payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_refund(self, async_client: AsyncWhop) -> None:
        payment = await async_client.payments.refund(
            id="pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_refund_with_all_params(self, async_client: AsyncWhop) -> None:
        payment = await async_client.payments.refund(
            id="pay_xxxxxxxxxxxxxx",
            partial_amount=6.9,
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_refund(self, async_client: AsyncWhop) -> None:
        response = await async_client.payments.with_raw_response.refund(
            id="pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_refund(self, async_client: AsyncWhop) -> None:
        async with async_client.payments.with_streaming_response.refund(
            id="pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_refund(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.payments.with_raw_response.refund(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retry(self, async_client: AsyncWhop) -> None:
        payment = await async_client.payments.retry(
            "pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retry(self, async_client: AsyncWhop) -> None:
        response = await async_client.payments.with_raw_response.retry(
            "pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retry(self, async_client: AsyncWhop) -> None:
        async with async_client.payments.with_streaming_response.retry(
            "pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retry(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.payments.with_raw_response.retry(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_void(self, async_client: AsyncWhop) -> None:
        payment = await async_client.payments.void(
            "pay_xxxxxxxxxxxxxx",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_void(self, async_client: AsyncWhop) -> None:
        response = await async_client.payments.with_raw_response.void(
            "pay_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_void(self, async_client: AsyncWhop) -> None:
        async with async_client.payments.with_streaming_response.void(
            "pay_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_void(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.payments.with_raw_response.void(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ForumPostListResponse", "User"]


class User(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class ForumPostListResponse(BaseModel):
    id: str
    """The unique identifier for the entity"""

    comment_count: int
    """The amount of comments on this post"""

    content: Optional[str] = None
    """The content of the forum post in Markdown format"""

    is_edited: bool
    """Whether the forum post has been edited"""

    is_pinned: bool
    """Whether this forum post is pinned"""

    is_poster_admin: bool
    """Whether the user that sent the post is an admin of the company"""

    like_count: Optional[int] = None
    """The number of likes this post has received"""

    parent_id: Optional[str] = None
    """The ID of the parent forum post, if applicable"""

    title: Optional[str] = None
    """The title of the forum post"""

    user: User
    """The user who created this forum post"""

    view_count: Optional[int] = None
    """The number of times this message has been viewed"""
